import type { NextApiRequest, NextApiResponse } from 'next';
/**
 *  an utility type to shorten writing Record<string, T = unknown>
 */
export declare type TypedObject<T = unknown> = Record<string, T>;
/**
 *  a sync/async function that takes next.js request and optional next.js response
 */
export declare type NextApiHandler<T = unknown, M extends TypedObject = TypedObject> = (req: NextApiRequestWithMiddleware<M>, res?: NextApiResponse) => T | Promise<T>;
/**
 *  a next.js api request with injected req.middleware
 */
export interface NextApiRequestWithMiddleware<T = TypedObject> extends NextApiRequest {
    middleware: T;
}
/**
 *  a standard next.js api handler but with req.middleware available
 *  see [official doc](https://nextjs.org/docs/api-routes/introduction) for more details
 */
export declare type NextApiHandlerWithMiddleware<T = unknown, M extends TypedObject = TypedObject> = (req: NextApiRequestWithMiddleware<M>, res: NextApiResponse<T>) => void | Promise<void>;
/**
 *  all api response combines success response & error response
 */
export declare type ApiResponse<T = unknown> = SuccessApiResponse<T> | ErrorApiResponse;
/**
 *  api response that has http status code 2xx with data
 */
export declare type SuccessApiResponse<T> = {
    success: true;
    data: T;
};
/**
 *  api response that has http status code 4xx or 5xx with error message
 */
export declare type ErrorApiResponse = {
    success: false;
    message: string;
};
/**
 * A router builder is next api handler builder that expose express-like api.
 *
 * ### Example (basic usage)
 * ```js
 * // In next.js /pages/api/[route]
 * import { RouterBuilder } from 'next-api-handler'
 * const handler = new RouterBuilder().build()
 * export default handler
 * ```
 *
 * ### Example (add RESTful method)
 * ```js
 * import { RouterBuilder } from 'next-api-handler'
 * const handler = new RouterBuilder().get((req, res) => "RESPONSE")
 * export default handler
 * ```
 *
 * ### Example (add async method)
 * ```js
 * import { RouterBuilder } from 'next-api-handler'
 * const handler = new RouterBuilder().get(async (req, res) => "ASYNC RESPONSE")
 * export default handler
 * ```
 *
 * @returns a builder that can build a next.js api handler.
 */
export declare class RouterBuilder {
    private readonly route;
    private readonly middlewareList;
    private readonly middlewareQueue;
    get<T = unknown, M extends TypedObject = TypedObject>(handler: NextApiHandler<T, M>): RouterBuilder;
    head<T = unknown, M extends TypedObject = TypedObject>(handler: NextApiHandler<T, M>): RouterBuilder;
    patch<T = unknown, M extends TypedObject = TypedObject>(handler: NextApiHandler<T, M>): RouterBuilder;
    options<T = unknown, M extends TypedObject = TypedObject>(handler: NextApiHandler<T, M>): RouterBuilder;
    connect<T = unknown, M extends TypedObject = TypedObject>(handler: NextApiHandler<T, M>): RouterBuilder;
    delete<T = unknown, M extends TypedObject = TypedObject>(handler: NextApiHandler<T, M>): RouterBuilder;
    trace<T = unknown, M extends TypedObject = TypedObject>(handler: NextApiHandler<T, M>): RouterBuilder;
    post<T = unknown, M extends TypedObject = TypedObject>(handler: NextApiHandler<T, M>): RouterBuilder;
    put<T = unknown, M extends TypedObject = TypedObject>(handler: NextApiHandler<T, M>): RouterBuilder;
    use<T extends TypedObject = TypedObject>(handler: NextApiHandler<T>): RouterBuilder;
    inject<T extends TypedObject = TypedObject>(handler: NextApiHandler<T>): RouterBuilder;
    build(): NextApiHandlerWithMiddleware;
    private handleMiddlewareQueue;
    private handleMiddlewareList;
    private add;
}
